@@ -9,6 +9,7 @@ from flask import Response, request, send_file
 from querystring_parser import parser
 
 from mlflow.entities import Metric, Param, RunTag, ViewType, ExperimentTag
+from mlflow.entities.model_registry import RegisteredModel, ModelVersion
 from mlflow.exceptions import MlflowException
 from mlflow.protos import databricks_pb2
 from mlflow.protos.service_pb2 import CreateExperiment, MlflowService, GetExperiment, \
@@ -16,51 +17,85 @@ from mlflow.protos.service_pb2 import CreateExperiment, MlflowService, GetExperi
     UpdateRun, LogMetric, LogParam, SetTag, ListExperiments, \
     DeleteExperiment, RestoreExperiment, RestoreRun, DeleteRun, UpdateExperiment, LogBatch, \
     DeleteTag, SetExperimentTag, GetExperimentByName
+from mlflow.protos.model_registry_pb2 import ModelRegistryService, CreateRegisteredModel, \
+    UpdateRegisteredModel, DeleteRegisteredModel, ListRegisteredModels, GetRegisteredModelDetails, \
+    GetLatestVersions, CreateModelVersion, UpdateModelVersion, DeleteModelVersion, \
+    GetModelVersionDetails, GetModelVersionDownloadUri, SearchModelVersions, GetModelVersionStages
 from mlflow.protos.databricks_pb2 import RESOURCE_DOES_NOT_EXIST
-from mlflow.store.artifact_repository_registry import get_artifact_repository
-from mlflow.store.dbmodels.db_types import DATABASE_ENGINES
-from mlflow.tracking.registry import TrackingStoreRegistry
+from mlflow.store.artifact.artifact_repository_registry import get_artifact_repository
+from mlflow.store.db.db_types import DATABASE_ENGINES
+from mlflow.tracking._model_registry.registry import ModelRegistryStoreRegistry
+from mlflow.tracking._tracking_service.registry import TrackingStoreRegistry
 from mlflow.utils.proto_json_utils import message_to_json, parse_dict
 from mlflow.utils.validation import _validate_batch_log_api_req
 
-_store = None
+_tracking_store = None
+_model_registry_store = None
 STATIC_PREFIX_ENV_VAR = "_MLFLOW_STATIC_PREFIX"
 
 
-def _add_static_prefix(route):
-    prefix = os.environ.get(STATIC_PREFIX_ENV_VAR)
-    if prefix:
-        return prefix + route
-    return route
+class TrackingStoreRegistryWrapper(TrackingStoreRegistry):
+    def __init__(self):
+        super(TrackingStoreRegistryWrapper, self).__init__()
+        self.register('', self._get_file_store)
+        self.register('file', self._get_file_store)
+        for scheme in DATABASE_ENGINES:
+            self.register(scheme, self._get_sqlalchemy_store)
+        self.register_entrypoints()
+
+    @classmethod
+    def _get_file_store(cls, store_uri, artifact_uri):
+        from mlflow.store.tracking.file_store import FileStore
+        return FileStore(store_uri, artifact_uri)
+
+    @classmethod
+    def _get_sqlalchemy_store(cls, store_uri, artifact_uri):
+        from mlflow.store.tracking.sqlalchemy_store import SqlAlchemyStore
+        return SqlAlchemyStore(store_uri, artifact_uri)
 
 
-def _get_file_store(store_uri, artifact_uri):
-    from mlflow.store.file_store import FileStore
-    return FileStore(store_uri, artifact_uri)
+class ModelRegistryStoreRegistryWrapper(ModelRegistryStoreRegistry):
+    def __init__(self):
+        super(ModelRegistryStoreRegistryWrapper, self).__init__()
+        # Model Registry does not support file based stores
+        self.register('', lambda store_uri: None)
+        self.register('file', lambda store_uri: None)
+        for scheme in DATABASE_ENGINES:
+            self.register(scheme, self._get_sqlalchemy_store)
+        self.register_entrypoints()
+
+    @classmethod
+    def _get_sqlalchemy_store(cls, store_uri):
+        from mlflow.store.model_registry.sqlalchemy_store import SqlAlchemyStore
+        return SqlAlchemyStore(store_uri)
 
 
-def _get_sqlalchemy_store(store_uri, artifact_uri):
-    from mlflow.store.sqlalchemy_store import SqlAlchemyStore
-    return SqlAlchemyStore(store_uri, artifact_uri)
+_tracking_store_registry = TrackingStoreRegistryWrapper()
+_model_registry_store_registry = ModelRegistryStoreRegistryWrapper()
 
 
-_tracking_store_registry = TrackingStoreRegistry()
-_tracking_store_registry.register('', _get_file_store)
-_tracking_store_registry.register('file', _get_file_store)
-for scheme in DATABASE_ENGINES:
-    _tracking_store_registry.register(scheme, _get_sqlalchemy_store)
-
-_tracking_store_registry.register_entrypoints()
-
-
-def _get_store(backend_store_uri=None, default_artifact_root=None):
+def _get_tracking_store(backend_store_uri=None, default_artifact_root=None):
     from mlflow.server import BACKEND_STORE_URI_ENV_VAR, ARTIFACT_ROOT_ENV_VAR
-    global _store
-    if _store is None:
+    global _tracking_store
+    if _tracking_store is None:
         store_uri = backend_store_uri or os.environ.get(BACKEND_STORE_URI_ENV_VAR, None)
         artifact_root = default_artifact_root or os.environ.get(ARTIFACT_ROOT_ENV_VAR, None)
-        _store = _tracking_store_registry.get_store(store_uri, artifact_root)
-    return _store
+        _tracking_store = _tracking_store_registry.get_store(store_uri, artifact_root)
+    return _tracking_store
+
+
+def _get_model_registry_store(backend_store_uri=None):
+    from mlflow.server import BACKEND_STORE_URI_ENV_VAR
+    global _model_registry_store
+    if _model_registry_store is None:
+        store_uri = backend_store_uri or os.environ.get(BACKEND_STORE_URI_ENV_VAR, None)
+        _model_registry_store = _model_registry_store_registry.get_store(store_uri)
+    return _model_registry_store
+
+
+def initialize_backend_stores(backend_store_uri=None, default_artifact_root=None):
+    _get_tracking_store(backend_store_uri, default_artifact_root)
+    _get_model_registry_store(backend_store_uri)
 
 
 def _get_request_json(flask_request=request):
@@ -108,14 +143,6 @@ def catch_mlflow_exception(func):
     return wrapper
 
 
-def get_handler(request_class):
-    """
-    :param request_class: The type of protobuf message
-    :return:
-    """
-    return HANDLERS.get(request_class, _not_implemented)
-
-
 _TEXT_EXTENSIONS = ['txt', 'log', 'yaml', 'yml', 'json', 'js', 'py',
                     'csv', 'tsv', 'md', 'rst', 'MLmodel', 'MLproject']
 
@@ -125,7 +152,7 @@ def get_artifact_handler():
     query_string = request.query_string.decode('utf-8')
     request_dict = parser.parse(query_string, normalized=True)
     run_id = request_dict.get('run_id') or request_dict.get('run_uuid')
-    run = _get_store().get_run(run_id)
+    run = _get_tracking_store().get_run(run_id)
     filename = os.path.abspath(_get_artifact_repo(run).download_artifacts(request_dict['path']))
     extension = os.path.splitext(filename)[-1].replace(".", "")
     # Always send artifacts as attachments to prevent the browser from displaying them on our web
@@ -145,8 +172,8 @@ def _not_implemented():
 @catch_mlflow_exception
 def _create_experiment():
     request_message = _get_request_message(CreateExperiment())
-    experiment_id = _get_store().create_experiment(request_message.name,
-                                                   request_message.artifact_location)
+    experiment_id = _get_tracking_store().create_experiment(request_message.name,
+                                                            request_message.artifact_location)
     response_message = CreateExperiment.Response()
     response_message.experiment_id = experiment_id
     response = Response(mimetype='application/json')
@@ -158,7 +185,7 @@ def _create_experiment():
 def _get_experiment():
     request_message = _get_request_message(GetExperiment())
     response_message = GetExperiment.Response()
-    experiment = _get_store().get_experiment(request_message.experiment_id).to_proto()
+    experiment = _get_tracking_store().get_experiment(request_message.experiment_id).to_proto()
     response_message.experiment.MergeFrom(experiment)
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -169,7 +196,7 @@ def _get_experiment():
 def _get_experiment_by_name():
     request_message = _get_request_message(GetExperimentByName())
     response_message = GetExperimentByName.Response()
-    store_exp = _get_store().get_experiment_by_name(request_message.experiment_name)
+    store_exp = _get_tracking_store().get_experiment_by_name(request_message.experiment_name)
     if store_exp is None:
         raise MlflowException(
             "Could not find experiment with name '%s'" % request_message.experiment_name,
@@ -184,7 +211,7 @@ def _get_experiment_by_name():
 @catch_mlflow_exception
 def _delete_experiment():
     request_message = _get_request_message(DeleteExperiment())
-    _get_store().delete_experiment(request_message.experiment_id)
+    _get_tracking_store().delete_experiment(request_message.experiment_id)
     response_message = DeleteExperiment.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -194,7 +221,7 @@ def _delete_experiment():
 @catch_mlflow_exception
 def _restore_experiment():
     request_message = _get_request_message(RestoreExperiment())
-    _get_store().restore_experiment(request_message.experiment_id)
+    _get_tracking_store().restore_experiment(request_message.experiment_id)
     response_message = RestoreExperiment.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -205,7 +232,8 @@ def _restore_experiment():
 def _update_experiment():
     request_message = _get_request_message(UpdateExperiment())
     if request_message.new_name:
-        _get_store().rename_experiment(request_message.experiment_id, request_message.new_name)
+        _get_tracking_store().rename_experiment(request_message.experiment_id,
+                                                request_message.new_name)
     response_message = UpdateExperiment.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -217,7 +245,7 @@ def _create_run():
     request_message = _get_request_message(CreateRun())
 
     tags = [RunTag(tag.key, tag.value) for tag in request_message.tags]
-    run = _get_store().create_run(
+    run = _get_tracking_store().create_run(
         experiment_id=request_message.experiment_id,
         user_id=request_message.user_id,
         start_time=request_message.start_time,
@@ -234,8 +262,8 @@ def _create_run():
 def _update_run():
     request_message = _get_request_message(UpdateRun())
     run_id = request_message.run_id or request_message.run_uuid
-    updated_info = _get_store().update_run_info(run_id, request_message.status,
-                                                request_message.end_time)
+    updated_info = _get_tracking_store().update_run_info(run_id, request_message.status,
+                                                         request_message.end_time)
     response_message = UpdateRun.Response(run_info=updated_info.to_proto())
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -245,7 +273,7 @@ def _update_run():
 @catch_mlflow_exception
 def _delete_run():
     request_message = _get_request_message(DeleteRun())
-    _get_store().delete_run(request_message.run_id)
+    _get_tracking_store().delete_run(request_message.run_id)
     response_message = DeleteRun.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -255,7 +283,7 @@ def _delete_run():
 @catch_mlflow_exception
 def _restore_run():
     request_message = _get_request_message(RestoreRun())
-    _get_store().restore_run(request_message.run_id)
+    _get_tracking_store().restore_run(request_message.run_id)
     response_message = RestoreRun.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -268,7 +296,7 @@ def _log_metric():
     metric = Metric(request_message.key, request_message.value, request_message.timestamp,
                     request_message.step)
     run_id = request_message.run_id or request_message.run_uuid
-    _get_store().log_metric(run_id, metric)
+    _get_tracking_store().log_metric(run_id, metric)
     response_message = LogMetric.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -280,7 +308,7 @@ def _log_param():
     request_message = _get_request_message(LogParam())
     param = Param(request_message.key, request_message.value)
     run_id = request_message.run_id or request_message.run_uuid
-    _get_store().log_param(run_id, param)
+    _get_tracking_store().log_param(run_id, param)
     response_message = LogParam.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -291,7 +319,7 @@ def _log_param():
 def _set_experiment_tag():
     request_message = _get_request_message(SetExperimentTag())
     tag = ExperimentTag(request_message.key, request_message.value)
-    _get_store().set_experiment_tag(request_message.experiment_id, tag)
+    _get_tracking_store().set_experiment_tag(request_message.experiment_id, tag)
     response_message = SetExperimentTag.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -303,7 +331,7 @@ def _set_tag():
     request_message = _get_request_message(SetTag())
     tag = RunTag(request_message.key, request_message.value)
     run_id = request_message.run_id or request_message.run_uuid
-    _get_store().set_tag(run_id, tag)
+    _get_tracking_store().set_tag(run_id, tag)
     response_message = SetTag.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -313,7 +341,7 @@ def _set_tag():
 @catch_mlflow_exception
 def _delete_tag():
     request_message = _get_request_message(DeleteTag())
-    _get_store().delete_tag(request_message.run_id, request_message.key)
+    _get_tracking_store().delete_tag(request_message.run_id, request_message.key)
     response_message = DeleteTag.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -325,7 +353,7 @@ def _get_run():
     request_message = _get_request_message(GetRun())
     response_message = GetRun.Response()
     run_id = request_message.run_id or request_message.run_uuid
-    response_message.run.MergeFrom(_get_store().get_run(run_id).to_proto())
+    response_message.run.MergeFrom(_get_tracking_store().get_run(run_id).to_proto())
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
     return response
@@ -343,8 +371,8 @@ def _search_runs():
     experiment_ids = request_message.experiment_ids
     order_by = request_message.order_by
     page_token = request_message.page_token
-    run_entities = _get_store().search_runs(experiment_ids, filter_string, run_view_type,
-                                            max_results, order_by, page_token)
+    run_entities = _get_tracking_store().search_runs(experiment_ids, filter_string, run_view_type,
+                                                     max_results, order_by, page_token)
     response_message.runs.extend([r.to_proto() for r in run_entities])
     if run_entities.token:
         response_message.next_page_token = run_entities.token
@@ -362,7 +390,7 @@ def _list_artifacts():
     else:
         path = None
     run_id = request_message.run_id or request_message.run_uuid
-    run = _get_store().get_run(run_id)
+    run = _get_tracking_store().get_run(run_id)
     artifact_entities = _get_artifact_repo(run).list_artifacts(path)
     response_message.files.extend([a.to_proto() for a in artifact_entities])
     response_message.root_uri = _get_artifact_repo(run).artifact_uri
@@ -376,8 +404,8 @@ def _get_metric_history():
     request_message = _get_request_message(GetMetricHistory())
     response_message = GetMetricHistory.Response()
     run_id = request_message.run_id or request_message.run_uuid
-    metric_entites = _get_store().get_metric_history(run_id,
-                                                     request_message.metric_key)
+    metric_entites = _get_tracking_store().get_metric_history(run_id,
+                                                              request_message.metric_key)
     response_message.metrics.extend([m.to_proto() for m in metric_entites])
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
@@ -387,7 +415,7 @@ def _get_metric_history():
 @catch_mlflow_exception
 def _list_experiments():
     request_message = _get_request_message(ListExperiments())
-    experiment_entities = _get_store().list_experiments(request_message.view_type)
+    experiment_entities = _get_tracking_store().list_experiments(request_message.view_type)
     response_message = ListExperiments.Response()
     response_message.experiments.extend([e.to_proto() for e in experiment_entities])
     response = Response(mimetype='application/json')
@@ -407,13 +435,163 @@ def _log_batch():
     metrics = [Metric.from_proto(proto_metric) for proto_metric in request_message.metrics]
     params = [Param.from_proto(proto_param) for proto_param in request_message.params]
     tags = [RunTag.from_proto(proto_tag) for proto_tag in request_message.tags]
-    _get_store().log_batch(run_id=request_message.run_id, metrics=metrics, params=params, tags=tags)
+    _get_tracking_store().log_batch(run_id=request_message.run_id, metrics=metrics,
+                                    params=params, tags=tags)
     response_message = LogBatch.Response()
     response = Response(mimetype='application/json')
     response.set_data(message_to_json(response_message))
     return response
 
 
+def _wrap_response(response_message):
+    response = Response(mimetype='application/json')
+    response.set_data(message_to_json(response_message))
+    return response
+
+
+@catch_mlflow_exception
+def _create_registered_model():
+    request_message = _get_request_message(CreateRegisteredModel())
+    registered_model = _get_model_registry_store().create_registered_model(request_message.name)
+    response_message = CreateRegisteredModel.Response(registered_model=registered_model.to_proto())
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _get_registered_model_details():
+    request_message = _get_request_message(GetRegisteredModelDetails())
+    registered_model_detailed = _get_model_registry_store().get_registered_model_details(
+        RegisteredModel.from_proto(request_message.registered_model))
+    response_message = GetRegisteredModelDetails.Response(
+        registered_model_detailed=registered_model_detailed.to_proto())
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _update_registered_model():
+    request_message = _get_request_message(UpdateRegisteredModel())
+    new_name = None
+    new_description = None
+    if request_message.HasField("name"):
+        new_name = request_message.name
+    if request_message.HasField("description"):
+        new_description = request_message.description
+    registered_model = _get_model_registry_store().update_registered_model(
+        RegisteredModel.from_proto(request_message.registered_model),
+        new_name,
+        new_description)
+    response_message = UpdateRegisteredModel.Response(registered_model=registered_model.to_proto())
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _delete_registered_model():
+    request_message = _get_request_message(DeleteRegisteredModel())
+    _get_model_registry_store().delete_registered_model(
+        RegisteredModel.from_proto(request_message.registered_model))
+    return _wrap_response(DeleteRegisteredModel.Response())
+
+
+@catch_mlflow_exception
+def _list_registered_models():
+    _get_request_message(ListRegisteredModels())
+    registered_models_detailed = _get_model_registry_store().list_registered_models()
+    response_message = ListRegisteredModels.Response()
+    response_message.registered_models_detailed.extend([e.to_proto()
+                                                        for e in registered_models_detailed])
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _get_latest_versions():
+    request_message = _get_request_message(GetLatestVersions())
+    latest_versions = _get_model_registry_store().get_latest_versions(
+        RegisteredModel.from_proto(request_message.registered_model), request_message.stages)
+    response_message = GetLatestVersions.Response()
+    response_message.model_versions_detailed.extend([e.to_proto() for e in latest_versions])
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _create_model_version():
+    request_message = _get_request_message(CreateModelVersion())
+    model_version = _get_model_registry_store().create_model_version(request_message.name,
+                                                                     request_message.source,
+                                                                     request_message.run_id)
+    response_message = CreateModelVersion.Response(model_version=model_version.to_proto())
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _get_model_version_details():
+    request_message = _get_request_message(GetModelVersionDetails())
+    model_version_detailed = _get_model_registry_store().get_model_version_details(
+        ModelVersion.from_proto(request_message.model_version))
+    response_proto = model_version_detailed.to_proto()
+    response_message = GetModelVersionDetails.Response(model_version_detailed=response_proto)
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _update_model_version():
+    request_message = _get_request_message(UpdateModelVersion())
+    new_stage = None
+    new_description = None
+    if request_message.HasField("stage"):
+        new_stage = request_message.stage
+    if request_message.HasField("description"):
+        new_description = request_message.description
+    _get_model_registry_store().update_model_version(
+        ModelVersion.from_proto(request_message.model_version),
+        new_stage,
+        new_description)
+    return _wrap_response(UpdateModelVersion.Response())
+
+
+@catch_mlflow_exception
+def _delete_model_version():
+    request_message = _get_request_message(DeleteModelVersion())
+    _get_model_registry_store().delete_model_version(
+        ModelVersion.from_proto(request_message.model_version))
+    return _wrap_response(DeleteModelVersion.Response())
+
+
+@catch_mlflow_exception
+def _get_model_version_download_uri():
+    request_message = _get_request_message(GetModelVersionDownloadUri())
+    download_uri = _get_model_registry_store().get_model_version_download_uri(
+        ModelVersion.from_proto(request_message.model_version))
+    response_message = GetModelVersionDownloadUri.Response(artifact_uri=download_uri)
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _get_model_version_stages():
+    request_message = _get_request_message(GetModelVersionStages())
+    stages = _get_model_registry_store().get_model_version_stages(
+        ModelVersion.from_proto(request_message.model_version))
+    response_message = GetModelVersionStages.Response()
+    response_message.stages.extend(stages)
+    return _wrap_response(response_message)
+
+
+@catch_mlflow_exception
+def _search_model_versions():
+    request_message = _get_request_message(SearchModelVersions())
+    model_versions_detailed = _get_model_registry_store().search_model_versions(
+        request_message.filter)
+    response_message = SearchModelVersions.Response()
+    response_message.model_versions_detailed.extend([e.to_proto() for e in model_versions_detailed])
+    return _wrap_response(response_message)
+
+
+def _add_static_prefix(route):
+    prefix = os.environ.get(STATIC_PREFIX_ENV_VAR)
+    if prefix:
+        return prefix + route
+    return route
+
+
 def _get_paths(base_path):
     """
     A service endpoints base path is typically something like /preview/mlflow/experiment.
@@ -423,22 +601,33 @@ def _get_paths(base_path):
     return ['/api/2.0{}'.format(base_path), _add_static_prefix('/ajax-api/2.0{}'.format(base_path))]
 
 
+def get_handler(request_class):
+    """
+    :param request_class: The type of protobuf message
+    :return:
+    """
+    return HANDLERS.get(request_class, _not_implemented)
+
+
 def get_endpoints():
     """
     :return: List of tuples (path, handler, methods)
     """
-    service_methods = MlflowService.DESCRIPTOR.methods
-    ret = []
-    for service_method in service_methods:
-        endpoints = service_method.GetOptions().Extensions[databricks_pb2.rpc].endpoints
-        for endpoint in endpoints:
-            for http_path in _get_paths(endpoint.path):
-                handler = get_handler(MlflowService().GetRequestClass(service_method))
-                ret.append((http_path, handler, [endpoint.method]))
-    return ret
+    def get_service_endpoints(service):
+        ret = []
+        for service_method in service.DESCRIPTOR.methods:
+            endpoints = service_method.GetOptions().Extensions[databricks_pb2.rpc].endpoints
+            for endpoint in endpoints:
+                for http_path in _get_paths(endpoint.path):
+                    handler = get_handler(service().GetRequestClass(service_method))
+                    ret.append((http_path, handler, [endpoint.method]))
+        return ret
+
+    return get_service_endpoints(MlflowService) + get_service_endpoints(ModelRegistryService)
 
 
 HANDLERS = {
+    # Tracking Server APIs
     CreateExperiment: _create_experiment,
     GetExperiment: _get_experiment,
     GetExperimentByName: _get_experiment_by_name,
@@ -460,4 +649,19 @@ HANDLERS = {
     ListArtifacts: _list_artifacts,
     GetMetricHistory: _get_metric_history,
     ListExperiments: _list_experiments,
+
+    # Model Registry APIs
+    CreateRegisteredModel: _create_registered_model,
+    GetRegisteredModelDetails: _get_registered_model_details,
+    DeleteRegisteredModel: _delete_registered_model,
+    UpdateRegisteredModel: _update_registered_model,
+    ListRegisteredModels: _list_registered_models,
+    GetLatestVersions: _get_latest_versions,
+    CreateModelVersion: _create_model_version,
+    GetModelVersionDetails: _get_model_version_details,
+    DeleteModelVersion: _delete_model_version,
+    UpdateModelVersion: _update_model_version,
+    GetModelVersionDownloadUri: _get_model_version_download_uri,
+    GetModelVersionStages: _get_model_version_stages,
+    SearchModelVersions: _search_model_versions,
 }
